!*********************************************************************
! file name   : ccwelfare.f90
! programmer  : makoto nakajima (makoto@uiuc.edu)
! description : program to implement welfare analysis for 1st
!               bankruptcy paper (ccdefss.f90).
! history     : 040928: first version
!               041126: allows preference and hospital bills shocks
!               061113: current version. celaned up and made
!                       compatiblewith the new program: ccdefss.f90.
!*********************************************************************
program main

	!******* implicit variable declaration prohibited *******
	implicit none

	!******* parameters *******
	real(8),parameter:: sigma     = 2.00000000000000000000_8
	integer,parameter:: na        = 3448
	integer,parameter:: azero     = 2396
	integer,parameter:: nh        = 2
	integer,parameter:: nm        = 3
	integer,parameter:: np        = 2
	integer,parameter:: nn        = 2
	integer,parameter:: ns        = nm*np*nn
	integer,parameter:: ni        = ns*(na+na-azero+1)
	integer,parameter:: nx        = nm+np+nn+1
	integer,parameter:: ny        = 4
	integer,parameter,dimension(nh):: a0s=(/1,azero/)

	!******* labels *******
	character(len=15),dimension(nx):: xlabel
	character(len=15),dimension(ny):: ylabel

	!******* meta parameters *******
	integer,parameter:: din =20
	integer,parameter:: dout=21

	!******* exogenous variables *******
	real(8),dimension(na):: aval
	real(8),dimension(ni):: val0,val1,dist0
	integer,dimension(ni):: ihfun,isfun,iafun
	integer,dimension(ns):: smfun,spfun,snfun
	integer,dimension(nh,ns,na):: iifun
	real(8),dimension(nh,ns):: disttemp

	!******* outputs *******
	real(8),dimension(nx,ny):: cons,vote,meas

	!******* indexes *******
	integer:: h0,s0,m0,p0,n0,a0,i0,x0,y0,itemp
	real(8):: rtemp,cons0,vote0,meas0

	!******* set labels *******
	ylabel(1)=' h=1 and a>=0.0'
	ylabel(2)=' h=1 and a< 0.0'
	ylabel(3)=' h=2           '
	ylabel(4)=' total         '
	xlabel(1)=' m=1           '
	xlabel(2)=' m=2           '
	xlabel(3)=' m=3           '
	xlabel(4)=' p=1           '
	xlabel(5)=' p=2           '
	xlabel(6)=' n=1           '
	xlabel(7)=' n=2           '
	xlabel(8)=' total         '

	!******* set index converters *******
	i0=0
	do h0=1,nh
		do s0=1,ns
			do a0=a0s(h0),na
				i0=i0+1
				ihfun(i0)=h0
				isfun(i0)=s0
				iafun(i0)=a0
				iifun(h0,s0,a0)=i0
			end do
		end do
	end do
	s0=0
	do m0=1,nm
		do p0=1,np
			do n0=1,nn
				s0=s0+1
				smfun(s0)=m0
				spfun(s0)=p0
				snfun(s0)=n0
			end do
		end do
	end do

	!******* read files *******
	open(din,file='./ccdefss_val0.dat')
	do i0=1,ni
		read(din,*) val0(i0)
	end do
	close(din)
	open(din,file='./ccdefss_val1.dat')
	do i0=1,ni
		read(din,*) val1(i0)
	end do
	close(din)
	open(din,file='./ccdefss_dist0.dat')
	do a0=1,na
		read(din,*) itemp,rtemp,((disttemp(h0,s0),s0=1,ns),h0=1,nh)
		do h0=1,nh; do s0=1,ns
			if (h0==2 .and. a0<azero) cycle
			dist0(iifun(h0,s0,a0))=disttemp(h0,s0)
		end do; end do
	end do
	close(din)

	!******* loop for each individual state *******
	cons=0.0_8
	vote=0.0_8
	meas=0.0_8
	do h0=1,nh; do s0=1,ns; do a0=a0s(h0),na
		m0=smfun(s0)
		p0=spfun(s0)
		n0=snfun(s0)
		i0=iifun(h0,s0,a0)
		meas0=dist0(i0)
		cons0=(val1(i0)/val0(i0))**(1.0_8/(1.0_8-sigma))-1.0_8
		if (cons0>=0.0_8) then
			vote0=meas0
		else
			vote0=0.0_8
		end if
		if (h0==1 .and. a0>=azero) then
			y0=1
		else if (h0==1) then
			y0=2
		else
			y0=3
		end if
		x0=m0
		cons(x0,y0)=cons(x0,y0)+cons0*meas0
		vote(x0,y0)=vote(x0,y0)+vote0
		meas(x0,y0)=meas(x0,y0)+meas0
		x0=nm+p0
		cons(x0,y0)=cons(x0,y0)+cons0*meas0
		vote(x0,y0)=vote(x0,y0)+vote0
		meas(x0,y0)=meas(x0,y0)+meas0
		x0=nm+np+n0
		cons(x0,y0)=cons(x0,y0)+cons0*meas0
		vote(x0,y0)=vote(x0,y0)+vote0
		meas(x0,y0)=meas(x0,y0)+meas0
	end do; end do; end do

	!******* sum up *******
	do x0=1,nx
		cons(x0,ny)=sum(cons(x0,1:3))
		vote(x0,ny)=sum(vote(x0,1:3))
		meas(x0,ny)=sum(meas(x0,1:3))
	end do
	do y0=1,ny
		cons(nx,y0)=sum(cons(1:nm,y0))
		vote(nx,y0)=sum(vote(1:nm,y0))
		meas(nx,y0)=sum(meas(1:nm,y0))
	end do

	!******* normalize *******
	do x0=1,nx; do y0=1,ny
		if (meas(x0,y0)>0.0_8) then
			cons(x0,y0)=cons(x0,y0)/meas(x0,y0)
			vote(x0,y0)=vote(x0,y0)/meas(x0,y0)
		else
			cons(x0,y0)=0.0_8
			vote(x0,y0)=0.0_8
		end if
	end do; end do
	meas(:,:)=meas(:,:)/meas(nx,ny)

	!******* print out *******
	open(dout,file='./ccwelfare.out')
	write(dout,"('******* measure *******')")
	write(dout,"(15x,4(1x,a15))") ylabel(1:ny)
	do x0=1,nx
		write(dout,"(a15,4(1x,f15.12))") xlabel(x0),meas(x0,1:ny)
	end do
	write(dout,"(/,'******* cons gain *******')")
	write(dout,"(15x,4(1x,a15))") ylabel(1:ny)
	do x0=1,nx
		write(dout,"(a15,4(1x,f15.12))") xlabel(x0),cons(x0,1:ny)
	end do
	write(dout,"(/,'******* measure voted for reform *******')")
	write(dout,"(15x,4(1x,a15))") ylabel(1:ny)
	do x0=1,nx
		write(dout,"(a15,4(1x,f15.12))") xlabel(x0),vote(x0,1:ny)
	end do
	close(dout)

end program main
